/*
    Valtra Tractors
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#include <string.h>
#include "button.h"

Button::Button() {
	font = NULL;
	content = NULL;
	for (int i = 0; i < 3; i++) {
		surf[i] = NULL;
		offset[i] = IPoint(0,0);
		imgOffset[i] = IPoint(0,0);
	}
	contOffset = textOffset = IPoint(0,0);
	text = NULL;
	state = laststate = 0;
	SDL_Color tc = { 0x00, 0x00, 0x00 };
	textColor = tc;
	freeSurfaces = false;
}

Button::~Button() {
	if (text) delete [] text;
	if (freeSurfaces) {
		for (int i = 0; i < 3; i++) {
			for (int j = i+1; j < 3; j++) {
				if (surf[i]==surf[j])
					surf[j] = NULL;
			}
		}

		for (int i = 0; i < 3; i++)
			SDL_FreeSurface(surf[i]);
	}
}

void Button::setSurf(SDL_Surface *s1, SDL_Surface *s2, SDL_Surface *s3) {
	surf[0] = s1;
	surf[1] = s2;
	surf[2] = s3;
	setClickmap(s1);
}

void Button::setClickOffset(int x, int y) {
	offset[0] = offset[1] = IPoint(0,0);
	offset[2].x = x;
	offset[2].y = y;
}

void Button::setText(const char *text, int x, int y, TTF_Font *f) {
	setText(text);
	textOffset.x = x;
	textOffset.y = y;
	font = f;
}

void Button::setPos(int x, int y) {
	r.x = x;
	r.y = y;
}

void Button::setClickmap(SDL_Surface *surf) {
	r.w = surf->w;
	r.h = surf->h;
	clickmap.clear();
	if (SDL_MUSTLOCK(surf)) SDL_LockSurface(surf);
	Uint8 *ptr = (Uint8*) surf->pixels;
	int bpp = surf->format->BytesPerPixel;
	for (int y = 0; y < surf->h; y++) {
		for (int x = 0; x < surf->w; x++) {
			Uint32 pixel = 0;
#if SDL_BYTEORDER == SDL_LIL_ENDIAN
			for (int i = 0; i < bpp; i++)
				pixel |= (*ptr++)<<(i*8);
#else
			for (int i = bpp-1; i >= 0; i--)
				pixel |= (*ptr++)<<(i*8);
#endif
			Uint8 foo, alpha;
			SDL_GetRGBA(pixel,surf->format,&foo,&foo,&foo,&alpha);
			clickmap.push_back(alpha>0xF0);
		}
		ptr += surf->pitch - surf->w*bpp;
	}
	if (SDL_MUSTLOCK(surf)) SDL_UnlockSurface(surf);
}

bool Button::isClickable(int x, int y) {
	x -= r.x;
	y -= r.y;
	if (x<0) return false;
	if (y<0) return false;
	if (x>=r.w) return false;
	if (y>=r.h) return false;
	return clickmap[y*r.w+x];
}

void Button::setText(const char *t) {
	if (text) delete [] text;
	if (t==NULL) {
		text = NULL;
		return;
	}
	int l = strlen(t)+1;
	char *ptr = new char[l];
	strcpy(ptr,t);
	text = ptr;
}

void Button::getIds(SpriteEngine *se) {
	for (int i = 0; i < 3; i++)
		ids[i] = se->getNextId();
}

void Button::addSprites(SpriteEngine *se) {
	SDL_Rect rect = r;
	rect.x += imgOffset[state].x;
	rect.y += imgOffset[state].y;
	SDL_Surface *s = surf[state];
	if (s==NULL) s = surf[0];
	se->addSprite(ids[0],new Sprite(s,rect,state!=laststate));
	if (content) {
		rect = r;
		rect.x += contOffset.x+offset[state].x;
		rect.y += contOffset.y+offset[state].y;
		se->addSprite(ids[1],new Sprite(content,rect,false));
	}
	if (text && font) {
		int w, h;
		TTF_SizeText(font,text,&w,&h);
		rect = r;
		rect.x += textOffset.x+offset[state].x;
		rect.y += textOffset.y+offset[state].y;
		rect.w = w;
		rect.h = h;
		se->addSprite(ids[2],new Sprite(NULL,rect,false));
	}
}

void Button::updateSprites(SpriteEngine *se) {
	Sprite *s = se->getSprite(ids[2]);
	if (s==NULL) return;
	if (!s->update) return;
	s->img = TTF_RenderText_Blended(font,text,textColor);
}

void Button::doneDrawing(SpriteEngine *se) {
	laststate = state;
	Sprite *s = se->getSprite(ids[2]);
	if (s==NULL) return;
	if (s->img) SDL_FreeSurface(s->img);
}




